package listFW.factory;

import listFW.*;

/**
 * Manufactures concrete IMTList and INEList objects.  Has only one
 * instance referenced by CompositeListFactory.Singleton.
 * @author Dung X. Nguyen
 * @author Stephen B. Wong
 * @since Copyright 2004 by DXN and SBW - All rights reserved
 */
public class CompositeListFactory implements IListFactory {
    /**
     * Singleton Pattern
     */
    public static final CompositeListFactory Singleton = new CompositeListFactory();
    private CompositeListFactory() {
    }
    
    private final static IMTList MTSingleton = new IMTList (){
        
        final public Object execute(IListAlgo algo, Object inp) {
            return algo.emptyCase(this, inp);
        }
        
        public String toString() {
            return "()";
        }
    };
    
    /**
     * Creates an empty list.
     * @return an IMTList object.
     */
    public IMTList makeEmptyList() {
        return MTSingleton;
    }
    
    /**
     * Creates a non-empty list containing a given first and a given rest.
     * @param first a data object.
     * @param tail != null, the rest of the non-empty list to be manufactured.
     * @return an INEList object containing first and tail
     */
    public INEList makeNEList(final Object first, final IList tail) {
        return new INEList() {
            final public Object getFirst() {
                return first;
            }
            
            final public IList getRest() {
                return tail;
            }
            
            final public Object execute(IListAlgo algo, Object inp) {
                return algo.nonEmptyCase(this, inp);
            }
                        
            public String toString() {
                return (String)ToStringAlgo.Singleton.nonEmptyCase(this, null);
            }
        };
    }
}