/**
 * Priority Queue Container using the heap structure.
 * STUDENTS TO ADD A NEW update method.
 */ 
package rac;

import listFW.*;
import listFW.factory.*;

public class RACHeap implements IRAContainer {
    /*
     * Initialize _table to reference a single-element array of
     * Comparable.
     */
    private Comparable[] _table = new Comparable[1];
    private int _tableOccupancy = 0;
        
    /**
     * Clears the contents of the RAC, leaving it empty.
     *
     * Implemented by replacing the existing array with a new,
     * empty one.
     */
    public void clear() {
        _table = new Comparable[1];
        _tableOccupancy = 0;
    }
    
    /**
     * Returns true if the RAC is empty and false otherwise.
     */
    public boolean isEmpty() {
        return _tableOccupancy == 0;
    }
    
    /**
     * Return TRUE if the container is full; otherwise, return
     * FALSE. 
     */
    public boolean isFull() {
        return false;
    }
    
    /**
     * Returns an IList of the objects corresponding to the entire
     * contents of the RAC.
     */
    public IList elements(IListFactory lf) {
        IList l = lf.makeEmptyList();
        
        for (int i = _tableOccupancy - 1; i >= 0; i--)
            l = lf.makeNEList(_table[i], l);
        
        return l;
    }
    
    /**
     * Removes and returns the highest priority object from the RAC.
     */
    public Object get() {
        Object result = _table[0];
        _tableOccupancy--;
        if (_tableOccupancy > 0) {
            _table[0] = _table[_tableOccupancy];
            
            Heapifier.Singleton.siftDown(_table, 0, 0, _tableOccupancy - 1);
        }
        else
            _table[0] = null;
        
        return (result);
    }
    
    /**
     * Inserts the given object into the RAC.  Requires that the object
     * is Comparable.
     */
    public void put(Object input) {
        if (_tableOccupancy == _table.length) {
            
            Comparable[] newTable = new Comparable[2*_table.length];
            
            for (int i = 0; i < _table.length; i++)
                newTable[i] = _table[i];
            
            _table = newTable;
        }
        _table[_tableOccupancy] = (Comparable)input;
        _tableOccupancy++;
        
        Heapifier.Singleton.siftUp(_table, 0, _tableOccupancy - 1);
    }
    
    /**
     * Return the next element in this IRAContainer withour removing it.
     * @throw an Exception if this IRAContainer is empty.
     */
    public Object peek() {
        if (isEmpty()) {
            throw new IllegalStateException("peek(): RAC is empty!");
        }
        else {
            return _table[0];
        }
    }
    
    /**
     * Restore the heap property using the Heapifier's siftUp() and/or siftDown()
     * methods after the input's priority has changed.  
     * @param input the object whose priority has changed.
     * @throw an IllegalArgument exception if input is not in the heap.
     * See TestRACHeap for some examples.
     */
    public void update(Comparable input) {
        // TO DO
        // Hint: first locate input using equals, then use compareTo to compare
        // the priorities of input against the priorities of its parent and/or
        // children.
    }
}
